/*
Copyright (c) NAVER Corp.
name: @egjs/vue-imready
license: MIT
author: NAVER Corp.
repository: https://github.com/naver/egjs-imready
version: 1.0.0
*/
import ImReady from '@egjs/imready';
import { reactive, ref, onMounted, onBeforeUnmount } from 'vue';

/**
 * Vue hook to check if the images or videos are loaded.
 * @ko 이미지와 비디오들이 로드가 됐는지 체크하는 Vue hook.
 * @memberof VueImReady
 * @param - Vue ImReady's props <ko> Vue ImReady의 props.</ko>
 * @example
 * ```js
 * import { useImReady } from "@egjs/vue-imready";
 *
 * setup() {
 *   const im = useImReady({
 *         selector: "img",
 *     });
 *
 *   return {
 *      im,
 *   }
 * }
 * // {{im.readyCount}}
 * // &lt;div v-bind:ref="im.register()"&gt;&lt;/div&gt;
 * ```
 */

function useImReady(props = {}) {
  const {
    usePreReady = true,
    usePreReadyElement = true,
    useReady = true,
    useReadyElement = true,
    useError = true,
    selector = "",
    ...options
  } = props;
  const children = [];
  const state = reactive({
    preReadyCount: 0,
    readyCount: 0,
    errorCount: 0,
    totalErrorCount: 0,
    totalCount: 0,
    isPreReady: false,
    isReady: false,
    isPreReadyOver: false,
    hasError: false,

    register(ref) {
      return instance => {
        if (instance) {
          children.push(instance);
        }

        if (!ref) {
          return;
        }

        if (typeof ref === "function") {
          ref(instance);
        } else {
          ref.value = instance;
        }
      };
    }

  });
  const imRef = ref();
  onMounted(() => {
    const im = new ImReady(options);
    imRef.value = im;
    let checkedElements = children;

    if (selector) {
      checkedElements = checkedElements.reduce((prev, cur) => {
        return [...prev, ...cur.querySelectorAll(selector)];
      }, []);
    }

    im.check(checkedElements).on("error", e => {
      if (useError) {
        state.hasError = true;
        state.errorCount = e.errorCount;
        state.totalErrorCount = e.totalErrorCount;
      }
    }).on("preReadyElement", e => {
      if (usePreReadyElement) {
        state.preReadyCount = e.preReadyCount;
      }
    }).on("readyElement", e => {
      if (useReadyElement) {
        state.readyCount = e.readyCount;
        state.isPreReadyOver = e.isPreReadyOver;
      }
    }).on("preReady", () => {
      if (usePreReady) {
        state.isPreReady = true;
      }
    }).on("ready", () => {
      if (useReady) {
        state.isReady = true;
      }
    });
    state.preReadyCount = 0;
    state.readyCount = 0;
    state.errorCount = 0;
    state.totalErrorCount = 0;
    state.totalCount = im.getTotalCount();
    state.isReady = false;
    state.isPreReady = false;
    state.hasError = false;
    state.isPreReadyOver = false;
  });
  onBeforeUnmount(() => {
    imRef.value?.destroy();
  });
  return state;
}

/**
 * Vue hook to check if the images or videos are loaded. (only `usePreReadyElement`, `usePreReady` and `useError` are true)
 * @ko 이미지와 비디오들이 로드가 됐는지 체크하는 Vue hook.(`usePreReadyElement`, `usePreReady`와 `useError`만 true)
 * @memberof VueImReady
 * @param - Vue ImReady's props <ko>Vue ImReady의 props.</ko>
 * @example
 * ```js
 * import { usePreReadyElement } from "@egjs/vue-imready";
 *
 * setup() {
 *   const im = usePreReadyElement({
 *         selector: "img",
 *     });
 *
 *   return {
 *      im,
 *   }
 * }
 * // {{im.preReadyCount}}
 * // &lt;div v-bind:ref="im.register()"&gt;&lt;/div&gt;
 * ```
 */

function usePreReadyElement(props = {}) {
  return useImReady({
    useReady: false,
    useReadyElement: false,
    ...props
  });
}

/**
 * Vue hook to check if the images or videos are loaded. (only `useReadyElement`, `useReady` and `useError` are true)
 * @ko 이미지와 비디오들이 로드가 됐는지 체크하는 vue hook.(`useReadyElement`, `useReady`와 `useError`만 true)
 * @memberof VueImReady
 * @param - Vue ImReady's props <ko>Vue ImReady의 props.</ko>
 * @example
 * ```js
 * import { useReadyElement } from "@egjs/vue-imready";
 *
 * setup() {
 *   const im = useReadyElement({
 *         selector: "img",
 *     });
 *
 *   return {
 *      im,
 *   }
 * }
 * // {{im.readyCount}}
 * // &lt;div v-bind:ref="im.register()"&gt;&lt;/div&gt;
 * ```
 */

function useReadyElement(props = {}) {
  return useImReady({
    usePreReadyElement: false,
    usePreReady: false,
    ...props
  });
}

/**
 * Vue hook to check if the images or videos are loaded. (only usePreReady and useError are true)
 * @ko 이미지와 비디오들이 로드가 됐는지 체크하는 Vue hook.(usePreReady와 useError만 true)
 * @memberof VueImReady
 * @param - Vue ImReady's props <ko>Vue ImReady의 props.</ko>
 * @example
 * ```js
 * import { usePreReady } from "@egjs/vue-imready";
 *
 * setup() {
 *   const im = usePreReady({
 *         selector: "img",
 *     });
 *
 *   return {
 *      im,
 *   }
 * }
 * // {{im.isPreReady}}
 * // &lt;div v-bind:ref="im.register()"&gt;&lt;/div&gt;
 * ```
 */

function usePreReady(props = {}) {
  return useImReady({
    usePreReadyElement: false,
    useReadyElement: false,
    useReady: false,
    ...props
  });
}

/**
 * Vue hook to check if the images or videos are loaded. (only `useReady` and `useError` are true)
 * @ko 이미지와 비디오들이 로드가 됐는지 체크하는 vue hook.(`useReady`와 `useError`만 true)
 * @memberof VueImReady
 * @param - Vue ImReady's props <ko>Vue ImReady의 props.</ko>
 * @example
 * ```js
 * import { useReady } from "@egjs/vue-imready";
 *
 * setup() {
 *   const im = useReady({
 *         selector: "img",
 *     });
 *
 *   return {
 *      im,
 *   }
 * }
 * // {{im.isReady}}
 * // &lt;div v-bind:ref="im.register()"&gt;&lt;/div&gt;
 * ```
 */

function useReady(props = {}) {
  return useImReady({
    usePreReadyElement: false,
    usePreReady: false,
    useReadyElement: false,
    ...props
  });
}

export { useImReady, usePreReady, usePreReadyElement, useReady, useReadyElement };
//# sourceMappingURL=imready.esm.js.map
